part of 'menu_bloc.dart';

@immutable
abstract class MenuState extends Equatable {
  const MenuState();

  @override
  List<Object> get props => [];
}

class MenuInitial extends MenuState {
  const MenuInitial();
}

class MenuLoading extends MenuState {}

class MenuReady extends MenuState {
  const MenuReady();
}

class MenuError extends MenuState {
  final String message;
  const MenuError({required this.message});

  @override
  List<Object> get props => [message];
}
